package handler

import (
	"GtDataStore/app/cmd/organization/internal/logic/job"
	"GtDataStore/app/cmd/organization/internal/svc"
	"GtDataStore/app/model"
	"context"
	"github.com/zeromicro/go-zero/core/logx"
	"time"
)

func DeviceMf(ctx context.Context, task *job.Task, technologyName string) error {
	// 1. 查询所有的设备
	devices, err := findDeviceMf(task.Job.SvcCtx, int64(task.Id))
	if err != nil {
		return err
	}
	// 2. 获得点位信息
	//var wg sync.WaitGroup
	//wg.Add(len(devices))
	//for _, device := range devices {
	//	device := device
	//	go func() {
	//		defer wg.Done()
	//		if err := device.Items.FillCurrentValue(); err != nil {
	//			logx.Errorf("DeviceMf device.Items.FillCurrentValue error: %s", err.Error())
	//		}
	//	}()
	//}
	//
	//wg.Wait()

	// 3. 转换为存储对象
	workings, err := transDeviceMfData(devices)
	if err != nil {
		return err
	}
	// 4. 批量入库
	if _, err := task.Job.SvcCtx.WorkingMf.MultiInsert(context.Background(), workings); err != nil {
		logx.Errorf("DeviceMf task.Job.SvcCtx.WorkingMf.MultiInsert error: %s", err.Error())
	}

	return nil
}

func findDeviceMf(svcCtx *svc.ServiceContext, projectId int64) ([]model.DcDeviceBind, error) {
	if data := deviceBindCacheTable.GetCache(DEVICE_MF); data != nil {
		return data, nil
	}
	if devices, err := svcCtx.DeviceBind.FindByProjectIdDeviceType(context.Background(), projectId, DEVICE_MF); err != nil {
		logx.Infof("findDeviceMf not found devices")
		return nil, err
	} else {
		deviceBindCacheTable.SetCache(DEVICE_MF, devices, time.Now().Add(300*time.Second))
		return devices, nil
	}
}

func transDeviceMfData(datas []model.DcDeviceBind) ([]model.DcWorkingMf, error) {
	ts := make([]model.DcWorkingMf, len(datas))

	for i, data := range datas {
		ts[i] = model.DcWorkingMf{
			ProjectId:        data.ProjectId,
			DeviceCode:       data.DeviceCode,
			WaterTemperature: data.Items.GetItemFloat64Value("water_temperature"),
			FeedFlow:         data.Items.GetItemFloat64Value("feed_flow"),
			ConFlow:          data.Items.GetItemFloat64Value("con_flow"),
			ProductFlow:      data.Items.GetItemFloat64Value("product_flow"),
			FeedPressure:     data.Items.GetItemFloat64Value("feed_pressure"),
			ConPressure:      data.Items.GetItemFloat64Value("con_pressure"),
			ProductPressure:  data.Items.GetItemFloat64Value("product_pressure"),
			Tmp:              data.Items.GetItemFloat64Value("tmp"),
			Flux:             data.Items.GetItemFloat64Value("flux"),
			FeedWqTurbidity:  data.Items.GetItemFloat64Value("feed_wq_turbidity"),
			FeedWqPh:         data.Items.GetItemInt64Value("feed_wq_ph"),
			ProductWqPh:      data.Items.GetItemInt64Value("product_wq_ph"),
			FeedWqAl:         data.Items.GetItemFloat64Value("feed_wq_al"),
			ProductWqAl:      data.Items.GetItemFloat64Value("product_wq_al"),
			FeedWqFe:         data.Items.GetItemFloat64Value("feed_wq_fe"),
			ProductWqFe:      data.Items.GetItemFloat64Value("product_wq_fe"),
			FeedWqMn:         data.Items.GetItemFloat64Value("feed_wq_mn"),
			ProductWqMn:      data.Items.GetItemFloat64Value("product_wq_mn"),
			FeedWqSio2:       data.Items.GetItemFloat64Value("feed_wq_sio2"),
			ProductWqSio2:    data.Items.GetItemFloat64Value("product_wq_sio2"),
			FeedWqCod:        data.Items.GetItemFloat64Value("feed_wq_cod"),
			ProductWqCod:     data.Items.GetItemFloat64Value("product_wq_cod"),
			FeedWqP:          data.Items.GetItemFloat64Value("feed_wq_p"),
			ProductWqP:       data.Items.GetItemFloat64Value("product_wq_p"),
			Step:             data.Items.GetItemInt64Value("step"),
			FilterTime:       data.Items.GetItemFloat64Value("filter_time"),
			CTime:            time.Now(),
		}
	}

	return ts, nil
}
